/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup bootSup
 * @{
 *
 * \file bootLib.c
 *
 * Implementation of\ref intro_sec "weAutSys"' bootloader helper functions
 *
 * \ref intro_sec "weAutSys" comes with a \ref bootSup "serial bootloader"
 * according to Atmel Corporation application note AVR109 on
 * "Self-programming".
 *
 * This bootloader was implemented for and tested on some quite different
 * target platforms,  ref intro_secH "weAut_01" being on of them. It can
 * easily be adapted to all similar platforms respectively to all AVR Cs
 * with self-programming support.
 *
 * One development goal was the tight integration with the system /
 * application software. That means
 *
 * a) not re-doing all or part of the platform initialisation and <br />
 * b) re-using bootloader functions in application software, <br />
 * c) which can also be done for final (flash) bootloader variables.
 *
 * See file boot109.c and  <br />
  &nbsp; &nbsp;
  Albrecht Weinert<br /> &nbsp; &nbsp;
<a href="http://a-weinert.de/pub/AVRserBootl.pdf"
 >A serial bootloader for weAut_01, ArduinoMega and akin</a><br />
  <br />
 *
 * $Revision: 1 $
 * $Date: 2017-01-25 11:03:06 +0000 (Wed, 25 Jan 2017) $
 */
/** @} */ // Doxygen addtogroup bootSup
/** @} */ // Doxygen addtogroup weAutSys

// { for Eclipse C AVR (indexer) bug only (horrible hack start) -----
#include <avr/io.h>
#include <stdint.h>
#include "arch/config.h"
// for Eclipse C AVR (indexer) bug only (horrible hack end)     ----- }


#include "we-aut_sys/ll_common.h"
#include "bootloader/boot109.h"

#include <avr/boot.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>

#define _SET_LOCK_BITS(data) boot_lock_bits_set(~data)
#define _ENABLE_RWW_SECTION() boot_rww_enable()

#define _WAIT_FOR_SPM() boot_spm_busy_wait()

#define _FILL_TEMP_WORD(addr,data) boot_page_fill(addr, data)
#define _PAGE_WRITE(addr) boot_page_write(addr)


/*  Set the LED on / off */
void setTheLed(uint8_t state){
#if defined(RD_LED_PORT)
   if (RD_LED_INV state) {
      orPort(RD_LED_PORT, (1 << RD_LED_PIN));
   } else {
      andPort(RD_LED_PORT, ~(1 << RD_LED_PIN));
   }
#elif defined(GN_LED_PORT)
   if (GN_LED_INV state) {
      orPort(GN_LED_PORT, (1 << GN_LED_PIN));
   } else {
      andPort(GN_LED_PORT, ~(1 << GN_LED_PIN));
   }
#else
   // platform specifica may be changed / added here in further elifs
#  warning "set the yellow (or gn) LED undefined"
#endif

} // setTheLed(uint8_t)


/* Calculate the pre-scaler setting for a desired baud-rate
 *
 * The calculation is based on the the nominal quartz processor clock, i.e.
 * macro F_CPU. Most Arduinos use crystals of so low grade that one has to
 * measure and define it as (macro) shitCrystal in Hz.
 */
uint16_t uartPrescaler(uint32_t baudRate, uint8_t x2){
   baudRate += baudRate; // *2
   baudRate += baudRate; // *4
   baudRate += baudRate; // *8
   if(!x2)    baudRate += baudRate; // *16   (may get 17 bit for 115200)
#if ALLOW_shitCrysta && defined(shitCrystal)
  // set shitCristal to the measured wrong quartz frequency (e.g. 15941400 )
  // return (uint16_t)( ((uint32_t)(shitCrystal))/ (baudRate * (x2 ? 8 : 16) ) );
   // for 16 MHz 24 bit (div24)  would be enough
   return (uint16_t)div32by24(NULL, (uint32_t)(shitCristal), baudRate);
#else
   // FCPU may be 17 bit if 20000000
  // return (uint16_t)( ((uint32_t)(F_CPU))/ (baudRate * (x2 ? 8 : 16) ) );
  return (uint16_t)div32by24(NULL, (uint32_t)(F_CPU), baudRate);
  //  return (uint16_t)( ((uint32_t)(F_CPU)) / baudRate);
#endif
} // uartPrescaler(uint32_t,  uint8_t)


uint32_t uartBaud(uint16_t uartPresc, uint8_t x2){
   uartPresc += 1; // divide by (uartPresc + 1) *  (x2 ? 8 : 16) )
   uartPresc += uartPresc; // (+1)*2
   uartPresc += uartPresc; // ..*4
   uartPresc += uartPresc; //  *8
   if(!x2)    uartPresc += uartPresc; // *16  (may get 16 bit but not larger)
#if ALLOW_shitCrysta && defined(shitCrystal)
  // return ((uint32_t)(shitCrystal))
    //     / ((uint32_t)(  (uartPresc + 1) *  (x2 ? 8 : 16) )) ;
   return (uint16_t)div32by16(NULL, (uint32_t)(shitCrystal), uartPresc);
  // set shitCristal to the measured wrong quartz frequency (e.g. 15941400 )
#else
   // return ((uint32_t)(F_CPU)) / uartPresc;
         // ((uint32_t)(  (uartPresc + 1) *  (x2 ? 8 : 16) )) ;
   return (uint16_t)div32by16(NULL, (uint32_t)(F_CPU), uartPresc);
#endif
} // uartBaud(uint16_t,  uint8_t)


/*  Unsigned 16 Bit divide
 *
 *  This function is an unconfined unsigned 16 bit divide. <br />
 *  This function does in effect
 *    *rem  = dividend % divisor;
 *    return = dividend / divisor;
 *
 *  \param rem  pointer to remainder; if null the remainder is lost
 *  \param dividend the number to be divided
 *  \param divisor  the divisor
 *  \return the quotient \c divisor / \c dividend
 */
uint16_t div16(uint16_t * rem, uint16_t dividend, uint16_t divisor){
     // Hint (internal): symbols starting with __ and not __u won't make it
     // into (cross) symbol table boot109_<target>_ext.tab

   uint16_t dividend2 = 0;
   uint16_t resAcc = 1;
   asm volatile("__loopS_%=:"
         "lsl %A0" "\n\t"
         "rol %B0"  "\n\t"
         "rol %A1"   "\n\t"
         "rol %B1"    "\n\t"

        //  "brcs __doSub_%=" "\n\t" // carry out == is bigger (can't happen at 16/16)
         "cp  %A1, %A3" "\n\t"
         "cpc  %B1, %B3" "\n\t" // dividend2 - divisor
         "brcc __doSub_%=" "\n\t" // if carry out ==  is lower: stay (no substract)
         "lsl %A2" "\n\t" // << 0 into resAccu
         "rol %B2"  "\n\t"
         "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out
         "rjmp __copyRes_%=" "\n\t" // ready

         "__doSub_%=:"
         "sub  %A1, %A3" "\n\t"
         "sbc  %B1, %B3"  "\n\t" // dividend2 -= divisor
         "sec"   "\n\t"   // set 1 << into result
         "rol %A2" "\n\t"
         "rol %B2"  "\n\t"
         "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out

         "__copyRes_%=:"

        : "+r"(dividend), "+r"(dividend2), "+r"(resAcc)
        : "r"(divisor)
        );

   if(rem) *rem = dividend2;
   return resAcc;
} //  div16(uint16_t *, 2*uint16_t)


/*  Unsigned 32 Bit divide
 *
 *  This function does in effect
 *
 *    *rem  = dividend % divisor;
 *    return = dividend / divisor;
 *
 * \param rem  pointer to remainder; if null the remainder is lost
 * \param dividend the number to be divided
 * \param divisor  the divisor
 * \return the quotient \c divisor / \c dividend
 */
uint32_t div32(uint32_t * rem, uint32_t dividend, uint32_t divisor){
   uint32_t dividend2 = 0;
   uint32_t resAcc = 1;
   asm volatile("__loopS_%=:"
         "lsl %A0" "\n\t"
         "rol %B0"  "\n\t"
         "rol %C0"   "\n\t"
         "rol %D0"    "\n\t"
         "rol %A1" "\n\t"
         "rol %B1"  "\n\t"
         "rol %C1"    "\n\t"
         "rol %D1"     "\n\t"

        //  "brcs __doSub_%=" "\n\t" // carry out == is bigger (can't happen at 16/16)
         "cp  %A1, %A3" "\n\t"
         "cpc  %B1, %B3" "\n\t" // dividend2 - divisor
         "cpc  %C1, %C3"  "\n\t"
         "cpc  %D1, %D3"  "\n\t"
         "brcc __doSub_%=" "\n\t" // if carry out ==  is lower: stay (no substract)
         "lsl %A2" "\n\t" // << 0 into resAccu
         "rol %B2"  "\n\t"
         "rol %C2"   "\n\t"
         "rol %D2"    "\n\t"
         "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out
         "rjmp __copyRes_%=" "\n\t" // ready

         "__doSub_%=:"
         "sub  %A1, %A3" "\n\t"
         "sbc  %B1, %B3"  "\n\t" // dividend2 -= divisor
         "sbc  %C1, %C3"   "\n\t"
         "sbc  %D1, %D3"   "\n\t"
         "sec"   "\n\t"   // set 1 << into result
         "rol %A2" "\n\t"
         "rol %B2"  "\n\t"
         "rol %C2"   "\n\t"
         "rol %D2"    "\n\t"
         "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out

         "__copyRes_%=:"

        : "+r"(dividend), "+r"(dividend2), "+r"(resAcc)
        : "r"(divisor)
        );

   if(rem) *rem = dividend2;
   return resAcc;
} // div32(uint32_t *, 2*uint32_t)

/*  Universal unsigned 24 bit divide
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32. It also shares the 32 bit
 *  parameter and return types (as avr-gcc C has no 24 bit type).
 *  Nevertheless it just operates on and returns 24 bit unsigned numbers
 *  &mdash; just ignoring respectively setting 0 the upper byte.
 *
 *  So it is about twice as fast as the optimised \ref div32.
 *
 * \param rem  pointer to the 32 bit variable for the 24 bit remainder;
 *             if null the remainder is lost; upper byte will be 0.
 * \param dividend the 24 bit number to be divided (upper byte ignored)
 * \param divisor  the 24 bit divisor (upper byte ignored)
 * \return the 24 bit quotient \c divisor / \c dividend (upper byte 0)
 * \sa div16
 */
uint32_t div24(uint32_t * rem, uint32_t dividend, uint32_t divisor){
    uint32_t dividend2 = 0;
    uint32_t resAcc = 1;
    asm volatile("__loopS_%=:"
          "lsl %A0  ;  \n\t"
          "rol %B0  ;   \n\t"
          "rol %C0  ; shift dividend one left into dividend2   \n\t"
          "rol %A1" "\n\t"
          "rol %B1"  "\n\t"
          "rol %C1"    "\n\t"

         //  "brcs __doSub_%=" "\n\t" // carry out == is bigger (can't happen at 24/24)
          "cp  %A1, %A3" "\n\t"
          "cpc  %B1, %B3" "\n\t" // dividend2 - divisor
          "cpc  %C1, %C3"  "\n\t"
          "brcc __doSub_%=" "\n\t" // if carry out ==  is lower: stay (no substract)
          "lsl %A2" "\n\t" // << 0 into resAccu
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out
          "rjmp __copyRes_%=" "\n\t" // ready

          "__doSub_%=:"
          "sub  %A1, %A3" "\n\t"
          "sbc  %B1, %B3"  "\n\t" // dividend2 -= divisor
          "sbc  %C1, %C3"   "\n\t"
          "sec"   "\n\t"   // set 1 << into result
          "rol %A2" "\n\t"
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out

          "__copyRes_%=:"
          "eor %D2, %D2  ; upper byte of quotient  0   \n\t"
          "eor %D1, %D1  ; upper byte of remainder 0   \n\t"
         : "+r"(dividend), "+r"(dividend2), "+r"(resAcc)
         : "r"(divisor)
         );

    if(rem) *rem = dividend2;
    return resAcc;
 } // div24(uint32_t *, 2*uint32_t)

/*  Unsigned 32 bit divide by 24 bit
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32 respectively div24 or a mix
 *  of both. It shares the 32 bit parameter and return types (as avr-gcc C has
 *  no 24 bit type).
 *
 *  The divisor and remainder are 24 bit numbers (upper 32 bit byte ignored or
 *  set 0). The dividend and the quotient are 32 bit.
 *
 * \param rem  pointer to the 32 bit variable for the 24 bit remainder;
 *             if null the remainder is lost; upper byte will be 0.
 * \param dividend the 32 bit number to be divided
 * \param divisor  the 24 bit divisor (upper 32 bit byte ignored)
 * \return the 32 bit quotient \c divisor / \c dividend (upper byte 0)
 * \sa div16
 */
uint32_t div32by24(uint32_t * rem, uint32_t dividend, uint32_t divisor){
    uint32_t dividend2 = 0;
    uint32_t resAcc = 1;
    asm volatile("__loopS_%=:"
          "lsl %A0" "\n\t"
          "rol %B0"  "\n\t"
          "rol %C0"   "\n\t"
          "rol %D0  ; shift dividend one left into dividend2   \n\t"
          "rol %A1" "\n\t"
          "rol %B1"  "\n\t"
          "rol %C1"    "\n\t"
          "brcs __doSub_%=" "\n\t" // carry out == is bigger (can happen at 32/24)

          "cp  %A1, %A3" "\n\t"
          "cpc  %B1, %B3" "\n\t" // dividend2 - divisor
          "cpc  %C1, %C3"  "\n\t"
          "brcc __doSub_%=" "\n\t" // if carry out ==  is lower: stay (no substract)
          "lsl %A2" "\n\t" // << 0 into resAccu
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "rol %D2"    "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out
          "rjmp __copyRes_%=" "\n\t" // ready

          "__doSub_%=:"
          "sub  %A1, %A3" "\n\t"
          "sbc  %B1, %B3"  "\n\t" // dividend2 -= divisor
          "sbc  %C1, %C3"   "\n\t"
          "sec"   "\n\t"   // set 1 << into result
          "rol %A2" "\n\t"
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "rol %D2"    "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out

          "__copyRes_%=:"
          "eor %D1, %D1  ; upper byte of remainder 0   \n\t"

         : "+r"(dividend), "+r"(dividend2), "+r"(resAcc)
         : "r"(divisor)
         );

    if(rem) *rem = dividend2;
    return resAcc;
 } // div32by24(uint32_t *, 2*uint32_t)

/*  Unsigned 32 bit divide by 16 bit
 *
 *  This functions's effect is \code
 *    result (return)  = dividend / divisor;
 *    *rem  = dividend % divisor;
 *  \endcode
 *  This function is quite similar to \ref div32 respectively \ref div16 or a
 *  mix of both.
 *
 *  The divisor and remainder are 16 bit numbers. The dividend and the
 *  quotient are 32 bit.
 *
 *  It is about 35% faster than \ref div32 (which is quite a lot on a RICS
 *  without division).
 *
 * \param rem  pointer to the 16 bit remainder;
 *             if null the remainder is lost
 * \param dividend the 32 bit number to be divided
 * \param divisor  the 16 bit divisor
 * \return the 32 bit quotient \c divisor / \c dividend
 * \sa div24
 */
uint32_t div32by16(uint16_t * rem, uint32_t dividend, uint16_t divisor){
    uint32_t dividend2 = 0;
    uint32_t resAcc = 1;
    asm volatile("__loopS_%=:"
          "lsl %A0" "\n\t"
          "rol %B0"  "\n\t"
          "rol %C0"   "\n\t"
          "rol %D0  ; shift dividend one left into dividend2   \n\t"
          "rol %A1" "\n\t"
          "rol %B1"  "\n\t"
          "brcs __doSub_%=" "\n\t" // carry out == is bigger (can happen at 32/24)

          "cp  %A1, %A3" "\n\t"
          "cpc  %B1, %B3" "\n\t" // dividend2 - divisor
          "brcc __doSub_%=" "\n\t" // if carry out ==  is lower: stay (no substract)
          "lsl %A2" "\n\t" // << 0 into resAccu
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "rol %D2"    "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out
          "rjmp __copyRes_%=" "\n\t" // ready

          "__doSub_%=:"
          "sub  %A1, %A3" "\n\t"
          "sbc  %B1, %B3"  "\n\t" // dividend2 -= divisor
          "sec"   "\n\t"   // set 1 << into result
          "rol %A2" "\n\t"
          "rol %B2"  "\n\t"
          "rol %C2"   "\n\t"
          "rol %D2"    "\n\t"
          "brcc __loopS_%=" "\n\t" // loop while no carry (init 1) out

          "__copyRes_%=:"
         : "+r"(dividend), "+r"(dividend2), "+r"(resAcc)
         : "r"(divisor)
         );

    if(rem) *rem = dividend2;
    return resAcc;
 } // div32by16(uint16_t *, uint32_t, uint16_t)

//---------------------------------------------------------------------------


/*  Initialise the serial input (UART0)
 *
 *  UART0 is initialised in asynchronous mode for receiving and
 *  transmitting (full duplex).
 *
 *  \param baudRate the rate desired (usually 38400)
 *  \param x2 true: double speed (half receive samples)
 *  \param parity 0=false: none; odd , even value: odd, even
 *  \param stopBits 2: 2 stop bits, 1 [default]: 1 stop bit
 *  \param useInt true: use UART interrupts
 */
void initUART0(uint32_t baudRate, uint8_t x2,
                          uint8_t parity, uint8_t stopBits, uint8_t useInt){
   if (x2) {
      UCSR0A = (1 << U2X0) | (1 << TXC0);
   } else {
      UCSR0A = (1 << TXC0);
   }
   uint16_t baudDivide = uartPrescaler(baudRate,  x2);
   UBRR0 =  baudDivide;  // set baudrate prescaler

   UCSR0B = useInt
         ? (1 << RXEN0)  | (1 << TXEN0)  // enable receive and transmit
           | (1 << RXCIE0) | (1 << TXCIE0) // receive + transmit interrupts
         : (1 << RXEN0)  | (1 << TXEN0);  // enable receive and transmit

   uint8_t r0c = parity ? ( (parity & 1) ? 0x30 : 0x20  ) : 0;
   if (stopBits == 2) r0c |= 0x08;
#ifdef URSEL
   r0c |= (0x06 | (1 << URSEL)); // length 8 bit (would be so after HW reset)
#else
   r0c |= 0x06; // length 8 bit (would be so after HW reset)
#endif

   UCSR0C = r0c;
   //  uint8_t tmp;
   while ( UCSR0A & (1<<RXC0) ) { // flush UART reception
       useInt = UDR0;
   } // could also be done by setting receiver off and on again
   UCSR0A |= (1 << TXC0);          // delete TXCflag
} // uartInit(uint16_t, ... )


/*  Basic UART0 send one byte
 *
 *  This is a very basic send function for the initialised UART0. It uses
 *  spin waiting and hence is meant for situations without any threading
 *  support (or interrupt), only.
 *
 *  \param c the character or byte to be sent
 *  \sa recvSerByte
 */
void sendSerByte(uint8_t c){
   while ( !( UCSR0A & (1 << UDRE0)) ); // wait for trans.buf. empty
   UDR0 = c; // send it
} //  sendSerByte(uint8_t c)


/*  Basic UART0 send multiple bytes from flash
 *
 *  This is a very basic send function for the initialised UART. It uses spin
 *  waiting and hence is meant for situations without any threading support
 *  (or interrupt), as e.g. early initialisation phases or single-threaded
 *  bootloaders.
 *
 *  \param src pointer to the first character or byte to be sent in
 *          of the 0-terminated string, not null
 *  \sa recvSerByte
 */
void sendSerBytes_P(ADDR_T src){
   // if (!src) return;
   for(;;) {
      uint8_t c = getFlashByte(src++);
      if (!c) break;
      sendSerByte(c);
   }
} // sendSerBytes_P(ADDR_T)

/*  Basic UART0 send multiple bytes from RAM
 *
 *  This is a very basic send function for the initialised UART. It uses spin
 *  waiting and hence is meant for situations without any threading support
 *  (or interrupt), as e.g. early initialisation phases or bootloaders.
 *
 *  \param src pointer to the first character or byte to be sent in
 *          of the 0-terminated string, must not be null
 *  \sa recvSerByte
 *  \sa sendSerByte
 *  \sa sendSerBytes_P
 */
void sendSerBytes(char * src){
   // if (!src) return; removed: never null by docu
   for(;;) {
      uint8_t c = *src++;
      if (!c) break;
      sendSerByte(c);
   }
} // sendSerBytes(char *)

/*  Copy a string from flash memory to RAM
 *
 *  This function copies up to \c mxLen-1 characters from \c src to \c dst
 *  and appends a trailing 0. Returned is the address of the last character
 *  in \c dst modified (the 0's address).
 *
 *  The operation will stop after \c mxLen characters transferred resp.
 *  modified or if a terminating 0 if found in \c src.
 *
 *  \param dst the destination to modify (in RAM, not null!)
 *  \param src the source (string) to copy from (in flash memory, not null!)
 *  \param mxLen the maximum number of characters to modify in \c dst
 *         including the trailing 0 appended. The maximum advance of the
 *         return value to parameter \c dest
 *  \return dst + number of characters modified; NULL only if \c dst is NULL
 *  \sa getSomeCharsP(char*, prog_char*, uint8_t)
 */
char * copyChars_P(char * dst,  ADDR_T src,  uint8_t mxLen){
   if (!mxLen || !src || !dst) return dst;
   for(;;) {
      if(!--mxLen) {
         *dst  = 0;
         break;
      }
      char c = getFlashByte(src++);
      *dst = c;
      if (c) {
         ++dst;
      } else break;
   }
   return dst;
} // copyChars_P(car*, ADDR_T, uint8_t)



/*  Basic UART wait for a byte received
 *
 *  This is a very basic receive function for the initialised UART. It uses
 *  spin waiting for a received byte timeout of tOut/10 seconds.
 *
 *  Cause of this spin waiting this function is meant for situations without
 *  any threading support (or interrupt), as e.g. early initialisation phases
 *  or bootloaders.
 *
 *  \param tOut a time out in 100 ms (1..254)
 *  \return 0: a byte was received; not 0: timeout
 *  \sa recvSerByte
 */
uint8_t waitSerByte(uint8_t tOut){
   while (!(UCSR0A & (1 << RXC0))){
      uint32_t locTimer = F_CPU / 109; // for 100ms (assuming 10 clocks/ spin)
      for(;;){
         if (UCSR0A & (1 << RXC0)) return 0;
         if(!--locTimer) { // check timout
            if (!--tOut) return 1;
            break; // 100 ms loop
         } // // check timout
      } // for 100 ms loop
   } // outer while UART receive not ready
   return 0; // ready
} // waitSerByte(uint8_t)

/*  Basic UART0 receive one byte
 *
 *  This is a very basic receive function for the initialised UART.
 *  It uses (endless) spin waiting!
 *
 *  \sa waitSerByte
 */
uint8_t recvSerByte(void){
   while(!(UCSR0A & (1 << RXC0)));  // wait for data
   return UDR0;
} // recvSerByte(void)

/*  Get UART receive error status and flush receiver on error
 *
 *  This function retrieves and returns the error status of an
 *  \ref initUART0 "initialised" UART(0). If no error occurred 0 is returned
 *  and nothing else done.
 *
 *  Otherwise the \ref initUART0 "initialised" UART's receiver is flushed.
 *
 *  \return 0 on no error or at least one of bits 4 (framing error), 3
 *          (overrun error) or 2 (parity error) is set
 */
uint8_t recvErrorState(void){
   uint8_t val = UCSR0A;
   uint8_t ret = val & 0x1C; // Bits 432 masked
   if (ret  && (val & (1 << RXC0))) do { // error and data
      val =  UDR0; // dummy read (should be max. twice)
   } while (UCSR0A & (1 << RXC0));
   return ret;
} // recvErrorState()


/*  A very basic delay function without any choice
 *
 *  Calling this function will keep the CPU busy for 500 cycles (two less for
 *  small flash devices) on a 20 MHz platform (weAut_01). That will take 25s.
 *  On slower platforms with e.g. 16 MHz (Arduino) the number of cycles is
 *  proportionally reduced to keep the 25 s.
 */
void wait25(void){
   uint8_t lc = //  130 for 16MHz; 245 for 20MHz
         ((F_CPU * 25 / 1000000) - 10) / 3;  // 25 s; -10 extraLoop
   for(;--lc;){ asm(""); } // 3 CPU clocks per iteration
} // wait25()

// internal development hint: If the only change is a modification of
// (one of) the following texts, try to keep the length unchanged.
// Otherwise the linkage of application to bootloader changes.
// The same applies for some (make defined) macros getting part of those
// texts.

/* Bootloader's welcome greeting and Copyright */
INFLASH(char const bootloaderWlc[]) = "\n\n \n"
    " * *   Welcome to weAutSys' bootloader    * * \n \n"
    " Protocol: Atmel AVR109, serial link       \n"
    " Copyright (c) 2015  weinert - automation  \n"
    " Prof. Dr.-Ing. Albrecht Weinert,  Bochum  \n \n";

INFLASH(char const  bootAut[]) = SYST_AUT;
INFLASH(char const  bootCop[]) = SYST_COP;
INFLASH(char const  systBld[]) = SYST_BLD;

INFLASH(char const  bootloaderPlatf[]) = " Platform: " PLATFORM_S
                ", CPU clock " FCPU_S "Hz   \n";

INFLASH(char const  bootTextRevis[]) =   " Revision:";
INFLASH(char const  bootTextReset[]) =   " Reset by:";


INFLASH(char const  bootResetCause1[]) = " power on  "; // reset cause: power on
INFLASH(char const  bootResetCause2[]) = " external reset "; // reset cause: external
INFLASH(char const  bootResetCause4[]) = " power (brown) out "; // reset cause: brown out
INFLASH(char const  bootResetCause8[]) = " watchdog reset "; // reset cause: watchdog
INFLASH(char const  bootResetCauseG[]) = " jtag reset "; // reset cause: JTAG
/* no reset cause: exit from from active bootloader or by command */
INFLASH(char const  bootResetCause0[]) = " no reset (by command?) ";

INFLASH(char const  bootResetCauseN[]) = " unknown cause "; // reset cause: not known

/*  The reset cause text
 *
 * \param resetCauses The C's reset info as by MCUSR
 * \return a 32 or 16 bit pointer (\ref ADDR_T) to a 0-terminated String in
 *         high (bootloader) flash memory
 */
ADDR_T resetCauseText_P(uint8_t resetCauses){
   if (! resetCauses)    return FAR_ADD(bootResetCause0);
   if (resetCauses & 8)  return FAR_ADD(bootResetCause8);
   if (resetCauses & 16) return FAR_ADD(bootResetCauseG);
   if (resetCauses & 1)  return FAR_ADD(bootResetCause1);
   if (resetCauses & 2)  return FAR_ADD(bootResetCause2);
   if (resetCauses & 4)  return FAR_ADD(bootResetCause4);
   return FAR_ADD(bootResetCauseN);
} //  resetCauseText_P(uint8_t)


/** Greeting for empty application flash  */
INFLASH(char const greetEmptFlash[]) =
  " \n \n remain in bootloader: application flash empty \n";


/* An eight bit count down display pattern
 *
 *  For every second counted down there is a 8bit / 8 LED display
 *  pattern the last one being 0 / all OFF. The total length is 72 (for
 *  72 seconds. For less than 72 seconds start at pattern
 *  &timOutCountPatr[72 - secs].
 */
INFLASH(uint8_t const timOutCountPatr[]) = {
      0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01, // 8
      0x81, 0x41, 0x21, 0x11, 0x09, 0x05, 0x03,      // 7
      0x83, 0x43, 0x23, 0x13, 0x0B, 0x07,           // 6
      0x87, 0x47, 0x27, 0x17, 0x0F,                // 5
      0x8F, 0x4F, 0x2F, 0x1F,                     // 4
      0x9F, 0x5F, 0x3F,                          // 3
      0xBF, 0x7F,                               // 2   (sum 35)
      0xFF, 0x7F, 0xBF, 0xDF, 0xEF, 0xF7, 0xFB, 0xFD,  // 8
      0xFE, 0x7E, 0xBE, 0xDE, 0xEE, 0xF6, 0xCA,       // 7
      0xFC, 0x7C, 0xBC, 0xDC, 0xEC, 0xF4,            // 6
      0xF8, 0x78, 0xB8, 0xD8, 0xE8,                 // 5
      0xF0, 0x70, 0xB0, 0xD0,                      // 4
      0xE0, 0x60, 0xA0,                           // 3
      0xC0, 0x40,                                // 2 (sum 35)
      0x80, 0x00, 0 };   // 2 +  1 * 0 for safety (length 72 +1)


#if defined(weAut_)

/*  De-select all SPI 1 devices with wait */
void deSelectAllSPI1(void){
   uint8_t retV = SPSR;
   if (!(retV & (1<<SPIF))) {
      uint8_t mxW = 4; // assumes max. SPI1 clock rate (16 CPU clocks / byte
      // Hint less than 4 is optimized loop flat and hence to short
      while (!(SPSR & (1<<SPIF)) && --mxW); // wait with timeout 6 clock loop
   }
   orCP( (1<<3) | (1<<2) );
} // deSelectAllSPI1()

#endif  // defined(weAut_)


/*  Output to a line of eight HMI / visible LEDs
 *
 *  If the target hardware has a line of eight LEDs this function sets them
 *  according to parameter \c val. Otherwise nothing is done.
 *
 *  With \ref intro_secH "weAut_01" these are the eight green digital input
 *  (DI) display LEDs.
 */
void toHMI8LEDchain(uint8_t val){
#if defined(weAut_)
   // Port C C2 /CS DIleds; C3 /CS DO
   andOrCP(~ (1<<2),   (1<<3) );
   SPDR = val;
   deSelectAllSPI1();
#elif defined(CHAIN8LED_PORT)
   setPort(CHAIN8LED_PORT, val);
#elif defined(NO_CHAIN8LED)
   // no eight LED port on UNO (no full usable 8DO port)
#else
   // in case of this warning: one of above macros (weAut_, CHAIN8LED_PORT or
   // NO_CHAIN8LED) must be defined in platform specific include
#  warning "toHMI8LEDchain undefined"
#endif
} // toHMI8LEDchain(uint8_t)


/*  Flash memory contains no application program
 *
 *  This function returns true if the flash cannot have been programmed
 *  with an application software after an erase.
 *
 *  The criterion is just flash[0] == 0xFFFF. It is simple and, yet, correct.
 */
uint8_t isFlashCleared(void){
   _WAIT_FOR_SPM();
   return pgm_read_word(0) == 0xFFFF;
} // isFlashCleared()

/*  Jump to normal main program
 *
 *  This function does some initialisations and jumps to address 0. That is
 *  the normal program's start in the flash memory.
 *
 *  \param init ON: do full \ref basicSystemInit() also
 */
void appMain(uint8_t init){
   _WAIT_FOR_SPM();
   if(init) {
      basicSystemInit();
   } else {
      wdt_disable(); cli();
      asm volatile ("clr __zero_reg__");
   }

   initUART0(STD_BAUD, OFF, 0, 1, USART_IN_APP_WITH_INT); // initialise UART

   _ENABLE_RWW_SECTION();

   asm volatile ("ldi r24,%0":: "M" (RAMEND & 0xFF));
   asm volatile ("ldi r25,%0":: "M" (RAMEND >> 8));
   asm volatile ("out __SP_H__,r25" ::);
   asm volatile ("out __SP_L__,r24" ::);

   asm volatile (""
         "clr r30" "\n\t"
         "clr r31" "\n\t"
         "ijmp" "\n\t"
          ::);
   while(1); // satisfy no return attribute
} // appMain()


/*  Jump to the bootloader
 *
 *  This function first disables interrupts and the watchdog.
 *
 *  With freshly set zeroReg and stackpointer the function then goes to
 *  the bootloader's start in the flash memory. Hence this function
 *  will never return to the caller.
 */
void bootMain(void){
   wdt_disable();  cli();        // disable watchdog for initialisation
   asm volatile ("clr __zero_reg__");
   asm volatile ("ldi r24,%0":: "M" (RAMEND & 0xFF));
   asm volatile ("ldi r25,%0":: "M" (RAMEND >> 8));
   asm volatile ("out __SP_H__,r25" ::);
   asm volatile ("out __SP_L__,r24" ::);

   ((void (*)())BOOTL_BEG)(); // enter the bootloader
   while(1);
} // bootMain(void)


/*  Write a sequence of bytes read from UART(0) to flash memory
 *
 *  This is a bootloader support function. It will read size bytes from the
 *  \ref recvSerByte "UART" and transfer them to the program memory starting
 *  at \c address. The \ref ADDR_T "address" the parameter points to must be
 *  a word address; it will be incremented by \c size/2 (if all goes well).
 *
 *  \param size the number of bytes to write to flash memory. It must be
 *              an even number
 *  \param address the start word address to write to in flash memory. (It is
 *              an even byte address.)
 *  \return address + size/2. That is the (incremented) address behind
 *              the last flash word modified
 */
ADDR_T blockLoadFl(uint16_t size, ADDR_T address){
   size &= 0xFFFE; // size must be even
   if (!size) return address;
   address <<= 1; // Convert to address to bytes
   ADDR_T tempaddress = address;  // Store address in page.
   do {
      ucnt16_t data;
      data.v8[0] = recvSerByte(); // word lo
      data.v8[1] = recvSerByte(); // word hi
      _FILL_TEMP_WORD(address, data.val);
      address += 2; // Select next word in memory.
      size -= 2; // Reduce number of bytes to write by two.
   } while(size); // Loop until all bytes written.

   _PAGE_WRITE(tempaddress);
   _WAIT_FOR_SPM();
   _ENABLE_RWW_SECTION();

   address >>= 1; // Convert address back to Flash words again.
   return address;
} //  blockLoadFl(uint16_t, ADDR_T*)

/*  Write a sequence of bytes read from UART(0) to EEPROM
 *
 *  This is a bootloader support function. It will read \c size bytes from
 *  the \ref recvSerByte "UART" and transfer them to the EEPROM memory
 *  starting at \c address. The \ref ADDR_T "address" parameter is
 *  a byte address.
 *
 *  \param size the number of bytes to write to EEPROM
 *  \param address the start address (of the first EEPROM byte to be modified).
 *         The parameters's type will fit the flash size even if EEPROMs
 *         are much smaller.
 *  \return address + size. That is the (incremented) address behind
 *              the last EEPROM byte modified
 */
ADDR_T blockLoadEE(uint16_t size, ADDR_T address){
   if (!size) return address;
   uint8_t buffer[size]; // a buffer (EEPROM is too slow to cope with UART)
   uint8_t *bufP = buffer;
   uint16_t sz = size;
   do {
      *bufP++ = recvSerByte();
   } while (--sz);
   bufP = buffer;
   sz = size;
   _WAIT_FOR_SPM();
   do {
      EEARL = ((ADDR_U)address).v8[0]; // Setup EEPROM address
      EEARH = ((ADDR_U)address).v8[1];
      EEDR = *bufP++;
      EECR |= (1<<EEMPE); // Write byte.
      EECR |= (1<<EEPE);
      while (EECR & (1<<EEPE)) ; // Wait for write operation to finish.
      address++; // Select next EEPROM byte
   }  while (--sz);
   return address;
} //  blockLoadEE(uint16_t, ADDR_T*)


/*  Read a sequence of bytes from EEPROM and send them via UART(0)
 *
 *  This is a bootloader support function. It will read size bytes from
 *  EEPROM starting at \c address and send them over the
 *  \ref recvSerByte "UART".
 *
 *  \param size the number of bytes to write to EEPROM memory
 *  \param address pointer to the source memory's start address. It will be
 *         incremented.
 *  \return address + size. That is the (incremented) address behind
 *              the last EEPROM byte read
 */
ADDR_T blockReadEE(uint16_t size, ADDR_T address){
   do  {
      EEAR = (uint16_t)address; // Setup EEPROM address
      ++address;
      EECR |= (1<<EERE); // Read EEPROM
      sendSerByte(EEDR); // Transmit EEPROM data to PC
   } while (--size);
   return address;
} // blockReadEE(uint16_t, ADDR_T)

/*  Read a sequence of bytes from flash memory and send them via UART(0)
 *
 *  This is a bootloader support function. It will read size bytes from
 *  flash memory starting at \c address and send them over the
 *  \ref recvSerByte "UART".  The \ref ADDR_T "address" parameter must be
 *  a word address. It value incremented by \c size/2 will be returned.
 *
 *  \param size the number of bytes to read from flash or program memory.
 *              (The size must be / will be made even.)
 *  \param address the start word address to read from flash memory.
 *              (It is, per def., an even byte address.)
 *  \return address + size/2. That is the (incremented) address behind
 *              the last flash word read
 */
ADDR_T blockReadFl(uint16_t size, ADDR_T address){
   size &= 0xFFFE; // size must be even
   if (!size) return address;
   address <<= 1; // Convert to address to bytes
   do  {
      sendSerByte( getFlashByte(address));  // even address
      // address += 1; // low byte or | 1 optim. will fail
      address |= 1;
      sendSerByte( getFlashByte(address));
      address += 1;
      size -= 2; // Subtract two bytes from number of bytes to read
   } while (size); // Repeat until all block has been read
   address >>= 1; // Convert address back to Flash words again.
   return address;
} // blockReadFl(uint16_t size, ADDR_T)
