/**
 * \file config_easyAVR32.h
 *
 *  Configuration settings for the easyAVR32 evaluation board
 *
 *  This file contains some platform specific definitions concerning software
 *  and hardware configuration. These settings influence the compilation and
 *  build process. Most of those settings can't be changed later at runtime.
 *
 *  Platform: easyAVR32; CPU: ATmega32;
 *  atmega32     # takes avr\iom32.h
 *
 *  Hardware specifica and settings:
 *
 *  Port A: set as input <br />
 *  Port B: 5..7= ISP <br />
 *  Port C: digital output
 *          used as HMI eight LED chain for bootloader and demo <br />
 *  Port D: 0,1 = R/TxD (UART); 7= LED (rd); 6= LED ("gn");
 *          5= TestPin0;; 4= Buzzer; 3= TestPin1 <br />
 *
 * This file is part of weAutSys &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 * <br />It is also used for the \ref bootSup "serial bootloader".
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2014  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * $Revision: 1 $
 * $Date: 2017-01-25 11:03:06 +0000 (Wed, 25 Jan 2017) $
 */
//  SVN last change by $Author: albrecht $

/** @} */ // Doxygen addtogroup   comTypes
/** @} */ // Doxygen addtogroup appLaySup

#ifndef CONFIG_PLATFORM_H
# define CONFIG_PLATFORM_H easyAVR32
#else
# error "more than one platform specific config_platform.h file"
#endif

//--- platform check for Eclipse    ----------------------------------------
#ifndef __AVR_ATmega32__
# error ECLIPSE warning: set target to atmega32 14745600
#endif

//--- platform specifica on ports ----------------------------------------

#define HAVE_NIC 0 //!< Network interface chip;  0: no switch \showinitializer
#define HAVE_SMC 0 //!< Small memory card;  0: no switch \showinitializer

#define HAVE_ETHERNET 0 //!< Ethernet available \showinitializer
#define HAVE_MODBUS 0 //!< Modbus on Ethernet available \showinitializer

#define UART0MAY_CTS 0  //!< UART0 may signal ability to receive by CTS \showinitializer
#define UART0MAY_RTS 0  //!< UART0 may use RTS to hold back sending \showinitializer
#define UART0SPIYKY 1 //!< UART0 has spiky USB2serial: filter FE,FF \showinitializer

#define GN_LED_PORT D //!< Port for yellow Status LED \showinitializer
#define GN_LED_PIN 7 //!< Pin for yellow Status LED \showinitializer
#define RD_LED_PORT D //!< Port reserved for red Status LED \showinitializer
#define RD_LED_PIN 6 //!< Pin reserved for red Status LED \showinitializer

#define BUZZER_PORT D //!< Port for buzzer (if any)  \showinitializer
#define BUZZER_PIN 4  //!< Pin for buzzer (if any) \showinitializer

#define CHAIN8LED_PORT C //!< Port for 8 independent LEDs in a row  \showinitializer

#ifndef STD_BAUD   // if not yet set set to default
# define STD_BAUD 19200  //!< Baudrate for easyAVR (can't handle 38400) \showinitializer
#endif

/** Port initialisation
 *
 *  This macro does the basic port initialisation according to the platform's
 *  standard port usage.
 *  Note for future changes: This macro may enable respectively initialise
 *  all ports and I/O functions in a way thought suitable for most application
 *  programmes. But it must always cover the serial bootloader's requirements.
 */
#define BASIC_INIT() do { \
   DDRA  = 0;    /* basically all input */ \
   PORTA = 0xFF; /*  pullups on  */         \
   DDRB = 0;     /* basically all input  */  \
   PORTB = 0xFF; /*  pullups on */            \
   DDRC  = 0xFF; /* Port C all out for  8 LEDs */ \
   PORTC = 0x81; /* outer LEDs on */               \
   DDRD = 0xFA;  /* 7/6 gn/rd LED; 4 buzzer; 5/3 TestPin0/1; D1 TXD out */ \
   PORTD = 0xC3; /* LEDs on TxD  1 resp. RxD  pull-up */  \
} while(0)

/** Report bootloader entrance by CLI on UART
 *
 *  Setting this to 1 we assume the UART on this platform being used for
 *  textual human interaction including command line interpreter (CLI).
 *  \showinitializer
 */
#define ALLOW_UART_GREET_BOOTLOADER_BY_CLI 1


/** Have and use a bootloader
 *
 *  This value determines the existence and usage of a bootloader: <br />
 *  0: no bootloader or at least no bootloader integration
 *  1: bootloader area used / bootloader existing
 *  2: bootloader fully integrated: application programme may use all
 *     No Read-While-Write (NRWW) flash section functions and constants.
 *
 *  Fully integrated (2) means that all Controller resets go through the
 *  bootloader. And the bootloader will not enter the program (=x00000)
 *  without having done all basic \ref BASIC_INIT() "initialisations", that
 *  hence can be omitted from system/application software. Full integration
 *  also means that the bootloader may be entered by (\ref shellCLI "CLI")
 *  command.
 *  \showinitializer
 */
#define USE_BOOTLOADER_PRESET 2

// no extra HW criterion (button) to enter / stay in bootloader
#define hwCritEntBoot() (0)

// Two bootloader settings (must either both or none be defined)
#define PROG_HW_VER "13" //!< The \ref programmerHWver \showinitializer
#define TIMEOUT_LIGHTSHOW 3 //!< Time out HMI display \showinitializer


// check easyAVR F_CPU
# ifndef F_CPU
#  define F_CPU 8000000
# elif  F_CPU != 8000000
// warning only because of the low grade Arduino crystals
#  warning "non standard MCU frequency specified for platform (check)"
# endif


// AVR32 has other port and register names
// as well as reduced capabilities compared to e.g. ATmega1284 or ATmega2560
// Hint: this part may be put to an extra .h file as soon as
// more platforms with low grade AVRs are being defined for
// the weAutSys' bootloader
#define UDR0 UDR
#define UCSR0A UCSRA
#define UCSR0B UCSRB
#define UCSR0C UCSRC
#define RXC0 RXC
#define RXEN0 RXEN
#define TXC0 TXC
#define TXCIE0 TXCIE
#define UBRR0 UBRRL
#define RXCIE0 RXCIE
#define TXEN0 TXEN
#define U2X0 U2X
#define UDRE0 UDRE

#define URSEL   7

#define EEMPE EEMWE
#define EEPE EEWE

/** Unable to XOR output pin
 *
 *  ATmega32 won't xor output port by writing to PINx
 *  \showinitializer
 */
#define CANT_OUT2PIN 1


